%This file defines the set of ODEs used to solve for electron and hole
%carrier concentrations. y(1,2,3,4) = n n' p and p' respectively, where n
%and p are electron and hole concentration and ' denotes the first
%derivative. x here is the independent variable, the position from cathode to anode, 0 being the cathode. Please reference the MATLAB help menu for help with setting up
%ODE computations.
function dydx = odeBHJ(x,y,E)

KbT = 0.025; %eV
q = 1.6e-19; %C
epsilon = 8.85e-12; %F/m

global E %dismiss red warnings
global avgperm
global ni

% If entering a specific recombination coefficient, set specR = 1. specR
% defaults to slowest carrier limited Langevin recombination (reference [insert
% citation] for more information).
specR = 0;
% Enter recombination coefficient if specified (m^3/s).
Rcoeff = 1.1e-17;

if specR == 1,
    
    dydx = [y(2)
        (-heav(Gopt(x)-Rcoeff*(y(1)*y(3)-ni^2))*(Gopt(x)-Rcoeff*(y(1)*y(3)-ni^2))-un(pos(x))*y(2)*(avgperm/perm(pos(x)))*E-dun(pos(x))*dphidx(x)*y(1)*(avgperm/perm(pos(x)))*E-un(pos(x))*y(1)*efieldder(pos(x))*dphidx(x)-KbT*dun(pos(x))*dphidx(x)*y(2))/(KbT*un(pos(x)))
        y(4)
        (-heav(Gopt(x)-Rcoeff*(y(1)*y(3)-ni^2))*(Gopt(x)-Rcoeff*(y(1)*y(3)-ni^2))+up(pos(x))*y(4)*(avgperm/perm(pos(x)))*E+dup(pos(x))*dphidx(x)*y(3)*(avgperm/perm(pos(x)))*E+up(pos(x))*y(3)*efieldder(pos(x))*dphidx(x)-KbT*dup(pos(x))*dphidx(x)*y(4))/(KbT*up(pos(x)))];
else
    
    dydx = [y(2)
        (-heav(Gopt(x)-(q/(perm(pos(x))*epsilon))*(un(pos(x))*up(pos(x))/(un(pos(x))+up(pos(x))))*(y(1)*y(3)-ni^2))*(Gopt(x)-(q/(perm(pos(x))*epsilon))*(un(pos(x))*up(pos(x))/(un(pos(x))+up(pos(x))))*(y(1)*y(3)-ni^2))-un(pos(x))*y(2)*(avgperm/perm(pos(x)))*E-dun(pos(x))*dphidx(x)*y(1)*(avgperm/perm(pos(x)))*E-un(pos(x))*y(1)*efieldder(pos(x))*dphidx(x)-KbT*dun(pos(x))*dphidx(x)*y(2))/(KbT*un(pos(x)))
        y(4)
        (-heav(Gopt(x)-(q/(perm(pos(x))*epsilon))*(un(pos(x))*up(pos(x))/(un(pos(x))+up(pos(x))))*(y(1)*y(3)-ni^2))*(Gopt(x)-(q/(perm(pos(x))*epsilon))*(un(pos(x))*up(pos(x))/(un(pos(x))+up(pos(x))))*(y(1)*y(3)-ni^2))+up(pos(x))*y(4)*(avgperm/perm(pos(x)))*E+dup(pos(x))*dphidx(x)*y(3)*(avgperm/perm(pos(x)))*E+up(pos(x))*y(3)*efieldder(pos(x))*dphidx(x)-KbT*dup(pos(x))*dphidx(x)*y(4))/(KbT*up(pos(x)))];
    
end